from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

class Piece(models.Model):

    class Meta:
        verbose_name = _('Piece')
        verbose_name_plural = _('Pieces')
        ordering = ['title']

    title = models.CharField(_('Title'), max_length=50)
    reference = models.CharField(_('Reference'), max_length=50, blank=True)
    comment = models.TextField(_('Comment'), blank=True)
    location = models.TextField(_('Location'), blank=True)
    image = models.ImageField(_('Picture'),
            upload_to='matos', max_length=250, null=True, blank=True)

    in_use = models.BooleanField(_('In use'), default=False)
    rentable = models.BooleanField(_('Rentable'), default=False)
    quantity = models.IntegerField(_('Quantity'), default=1, null=True)

    purchase_date = models.DateField(_('Purchase Date'), null=True, blank=True)
    purchase_price = models.DecimalField(_('Purchase Price'),
            max_digits=10, decimal_places=2, null=True, blank=True)

    creation_timestamp = models.DateTimeField(auto_now_add=True, null=True)
    last_update_timestamp = models.DateTimeField(auto_now=True, null=True)

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('piece-view', kwargs={'pk': self.id})


class Loan(models.Model):

    class Meta:
        verbose_name = _('Loan')
        verbose_name_plural = _('Loans')

    date_start = models.DateField(null=True)
    date_end = models.DateField(null=True)
    comment = models.TextField(blank=True)

    loaner = models.CharField(_('Loaner'), blank=True, max_length=50)
    pieces = models.ManyToManyField(Piece, verbose_name=_('Pieces'),
            null=True, blank=True, through='LoanPiece')

    def get_absolute_url(self):
        return reverse('loan-view', kwargs={'pk': self.id})


class LoanPiece(models.Model):
    piece = models.ForeignKey(Piece)
    loan = models.ForeignKey(Loan)
    quantity = models.IntegerField(_('Quantity'), default=1)
