from django.core.urlresolvers import reverse

from django.views.generic.list import ListView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView, UpdateView, DeleteView

from .models import Piece, Loan, LoanPiece
from .forms import LoanForm


class PieceListView(ListView):
    model = Piece


class PieceDetailView(DetailView):
    model = Piece


class PieceCreateView(CreateView):
    model = Piece


class PieceUpdateView(UpdateView):
    model = Piece


class PieceDeleteView(DeleteView):
    model = Piece

    def get_success_url(self):
        return reverse('pieces-list')


class LoanCreateView(CreateView):
    model = Loan
    form_class = LoanForm

    def get_context_data(self, *args, **kwargs):
        context = super(LoanCreateView, self).get_context_data(*args, **kwargs)
        context['pieces'] = Piece.objects.filter(rentable=True)
        return context


class LoanDetailView(DetailView):
    model = Loan


class LoanUpdateView(UpdateView):
    model = Loan
    form_class = LoanForm

    def form_valid(self, form):
        pieces = {}
        for piece in self.request.POST.getlist('id_pieces'):
            if piece not in pieces:
                pieces[piece] = 0
            pieces[piece] += 1
        result = super(LoanUpdateView, self).form_valid(form)
        self.object.pieces.clear()
        for piece_id, quantity in pieces.items():
            LoanPiece.objects.create(piece=Piece.objects.get(id=piece_id),
                    loan=self.object, quantity=quantity)
        return result

    def get_context_data(self, *args, **kwargs):
        context = super(LoanUpdateView, self).get_context_data(*args, **kwargs)
        context['pieces'] = Piece.objects.filter(rentable=True)
        context['loaned'] = []
        for x in LoanPiece.objects.filter(loan=self.object):
            context['loaned'].extend([x.id]*x.quantity)
        return context


class LoanDeleteView(DeleteView):
    model = Loan

    def get_success_url(self):
        return reverse('pieces-list')
